﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Microsoft.Build.Utilities;
using Microsoft.Build.Framework;

namespace Microsoft.Tools.CodeRunner.Tasks
{
    public class Csr : Task
    {
        private string scriptPath;
        private string arguments;
        private bool? debugMessages;
        private bool? stackTraces;
        private bool? searchSystemPath;
        private bool? allowUnsafeCode;
        private int exitCode = 0;

        [Required] 
        public string ScriptPath 
        { 
            get { return scriptPath; } 
            set { scriptPath = value; } 
        }
        
        public string Arguments 
        { 
            get { return arguments; }
            set { arguments = value; }
        }
        
        public bool DebugMessages 
        { 
            get { return debugMessages ?? false; }
            set { debugMessages = value; } 
        }
        
        public bool StackTraces
        { 
            get { return stackTraces ?? false; }
            set { stackTraces = value; } 
        }
        
        public bool SearchSystemPath
        { 
            get { return searchSystemPath ?? false; }
            set { searchSystemPath = value; } 
        }
        
        public bool AllowUnsafeCode
        {
            get { return allowUnsafeCode ?? false; }
            set { allowUnsafeCode = value; } 
        }

        [Output] 
        public int ExitCode { get; set; }

        public override bool Execute()
        {
            ProcessStartInfo si = new ProcessStartInfo("csr.exe");
            
            ConfigBuilder configBuilder = new ConfigBuilder();

            configBuilder.AppendIfNotNull("DebugMessages", debugMessages);
            configBuilder.AppendIfNotNull("StackTraces", stackTraces);
            configBuilder.AppendIfNotNull("SearchSystemPath", searchSystemPath);
            configBuilder.AppendIfNotNull("AllowUnsafeCode", allowUnsafeCode);
            
            if (configBuilder.Count > 0)
                si.EnvironmentVariables["CSR_CONFIG"] = configBuilder.ToString();
            
            si.Arguments = scriptPath + " " + arguments;

            Process process = null;
            bool ok = false;
            
            try 
            {
                process = Process.Start(si);
                
                if (process != null)
                {
                    process.WaitForExit();
                    ok = true;
                    exitCode = process.ExitCode;
                }
                else
                {
                    Log.LogErrorWithCodeFromResources("UnableToLaunch");
                }
            }
            finally
            {
                if (process != null)
                {
                    process.Close();
                    process = null;
                }
            }
            
            return ok;
        }
    }
}
