﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Microsoft.Tools.CodeRunner
{
    public sealed class ConsoleOutputter : IOutputter
    {
        #region Private Fields
        private TextWriter writer;

        #endregion

        #region Constructors
        public ConsoleOutputter()
        {
            writer = Console.Out;
        }

        public ConsoleOutputter(TextWriter writer)
        {
            this.writer = writer;
        }

        #endregion

        #region Public Properties
        public bool Silent { get; set; }

        #endregion

        #region IOutputter Implementation
        void IOutputter.OutputCustomEvent(OutputCustomEventArgs e)
        {
        }

        public void OutputErrorEvent(OutputErrorEventArgs e)
        {
            if (Silent)
                return;

            Console.ForegroundColor = ConsoleColor.Red;
            writer.Write(OutputterResources.Error, e.Code);
            writer.WriteLine(e.Message);
            Console.ResetColor();
        }

        public void OutputWarningEvent(OutputWarningEventArgs e)
        {
            if (Silent)
                return;

            Console.ForegroundColor = ConsoleColor.Yellow;
            writer.Write(OutputterResources.Warning, e.Code);
            writer.WriteLine(e.Message);
            Console.ResetColor();
        }

        public void OutputMessageEvent(OutputMessageEventArgs e)
        {
            if (Silent)
                return;

            switch (e.Importance)
            {
                case MessageImportance.Low:
                    Console.ForegroundColor = ConsoleColor.DarkGray;
                    break;

                case MessageImportance.High:
                    Console.ForegroundColor = ConsoleColor.Cyan;
                    break;

                default:
                    Console.ForegroundColor = ConsoleColor.Gray;
                    break;
            }

            writer.WriteLine(e.Message);
            Console.ResetColor();
        }
        #endregion
    }
}
